@@ -24,9 +24,6 @@
  * the next time I get bored
  */
 
-/* this file really should get split/cleaned up sometime ;) */
-#include "config.h"
-
 #if STDC_HEADERS
 # include <stdlib.h>
 #endif
@@ -38,20 +35,66 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <audacious/plugin.h>
-#include <audacious/auddrct.h>
+
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <pthread.h>
 #include <assert.h>
 #include <math.h>
 
+#include <audacious/debug.h>
+#include <audacious/drct.h>
+#include <audacious/misc.h>
+#include <audacious/plugin.h>
+
 #include "alarm.h"
 #include "interface.h"
 #include "callbacks.h"
 
-static pthread_t start_tid;	       /* thread id of alarm loop */
-static pthread_t stop_tid;	       /* thread id of stop loop */
+static const gchar * const alarm_defaults[] = {
+ /* general */
+ "alarm_h", "6",
+ "alarm_m", "30",
+ "cmd_on", "FALSE",
+ "fading", "60",
+ "quietvol", "25",
+ "reminder_on", "FALSE",
+ "stop_h", "1",
+ "stop_m", "0",
+ "stop_on", "TRUE",
+ "volume", "80",
+
+ /* days */
+ "sun_flags", "3",
+ "sun_h", "6",
+ "sun_m", "30",
+ "mon_flags", "2",
+ "mon_h", "6",
+ "mon_m", "30",
+ "tue_flags", "2",
+ "tue_h", "6",
+ "tue_m", "30",
+ "wed_flags", "2",
+ "wed_h", "6",
+ "wed_m", "30",
+ "thu_flags", "2",
+ "thu_h", "6",
+ "thu_m", "30",
+ "fri_flags", "2",
+ "fri_h", "6",
+ "fri_m", "30",
+ "sat_flags", "2",
+ "sat_h", "6",
+ "sat_m", "30",
+ NULL};
+
+typedef struct {
+    pthread_t tid;
+    volatile gboolean  is_valid;
+} alarm_thread_t;
+
+static alarm_thread_t start;    /* thread id of alarm loop */
+static alarm_thread_t stop;     /* thread id of stop loop */
 static pthread_mutex_t fader_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static GeneralPlugin alarm_plugin;
@@ -124,26 +167,12 @@ static GtkWidget *alarm_dialog = NULL;
 
 static GtkWidget *lookup_widget(GtkWidget *w, const gchar *name)
 {
-   GtkWidget *widget;
-
-   widget = (GtkWidget*) gtk_object_get_data(GTK_OBJECT(w),
-					     name);
+   GtkWidget * widget = g_object_get_data ((GObject *) w, name);
    g_return_val_if_fail(widget != NULL, NULL);
 
    return widget;
 }
 
-static void dialog_destroyed(GtkWidget *dialog, gpointer data)
-{
-   DEBUG("dialog destroyed\n");
-   *(GtkObject**)data = NULL;
-}
-
-static inline gboolean dialog_visible(GtkWidget *dialog)
-{
-   return(((dialog != NULL) && GTK_WIDGET_VISIBLE(dialog)));
-}
-
 /*
  * tell the user about that bug
  */
@@ -152,13 +181,13 @@ static void alarm_warning(void)
 
    static GtkWidget *warning_dialog = NULL;
 
-   if(dialog_visible(warning_dialog))
+   if (warning_dialog)
      return;
 
    warning_dialog = create_warning_dialog();
 
-   gtk_signal_connect(GTK_OBJECT(warning_dialog), "destroy",
-		      GTK_SIGNAL_FUNC(dialog_destroyed), &warning_dialog);
+   g_signal_connect (warning_dialog, "destroy", (GCallback)
+    gtk_widget_destroyed, & warning_dialog);
 
    gtk_widget_show_all(warning_dialog);
 
@@ -172,32 +201,19 @@ static void alarm_warning(void)
 void alarm_save(GtkButton *w, gpointer data)
 {
    int daynum = 0;  // used to identify day number
-   mcs_handle_t *conf;
-
-   DEBUG("alarm_save\n");
-
-   conf = aud_cfg_db_open();
 
    /*
     * update the live values and write them out
     */
-   alarm_h = alarm_conf.default_hour = 
-     gtk_spin_button_get_value_as_int(alarm_conf.alarm_h);
-   aud_cfg_db_set_int(conf, "alarm", "alarm_h", alarm_h);
+   alarm_h = alarm_conf.default_hour = gtk_spin_button_get_value_as_int (alarm_conf.alarm_h);
+   aud_set_int ("alarm", "alarm_h", alarm_h);
 
-   alarm_m = alarm_conf.default_min = 
-     gtk_spin_button_get_value_as_int(alarm_conf.alarm_m);
-   aud_cfg_db_set_int(conf, "alarm", "alarm_m", alarm_m);
+   alarm_m = alarm_conf.default_min = gtk_spin_button_get_value_as_int (alarm_conf.alarm_m);
+   aud_set_int ("alarm", "alarm_m", alarm_m);
 
-
-   stop_h =
-     gtk_spin_button_get_value_as_int( alarm_conf.stop_h);
-
-   stop_m =
-     gtk_spin_button_get_value_as_int(alarm_conf.stop_m);
-
-   stop_on =
-     gtk_toggle_button_get_active(alarm_conf.stop_on);
+   stop_h = gtk_spin_button_get_value_as_int (alarm_conf.stop_h);
+   stop_m = gtk_spin_button_get_value_as_int (alarm_conf.stop_m);
+   stop_on = gtk_toggle_button_get_active (alarm_conf.stop_on);
 
    /* days of the week */
    for(; daynum < 7; daynum++)
@@ -206,217 +222,141 @@ void alarm_save(GtkButton *w, gpointer data)
        alarm_conf.day[daynum].flags = 0;
      else
        alarm_conf.day[daynum].flags = ALARM_OFF;
-   
+
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(alarm_conf.day[daynum].cb_def)))
        alarm_conf.day[daynum].flags |= ALARM_DEFAULT;
 
-     alarm_conf.day[daynum].hour = 
+     alarm_conf.day[daynum].hour =
        gtk_spin_button_get_value_as_int(alarm_conf.day[daynum].spin_hr);
-     alarm_conf.day[daynum].min = 
+     alarm_conf.day[daynum].min =
        gtk_spin_button_get_value_as_int(alarm_conf.day[daynum].spin_min);
 
-     aud_cfg_db_set_int(conf, "alarm", day_flags[daynum], alarm_conf.day[daynum].flags);
-     aud_cfg_db_set_int(conf, "alarm", day_h[daynum], alarm_conf.day[daynum].hour);
-     aud_cfg_db_set_int(conf, "alarm", day_m[daynum], alarm_conf.day[daynum].min);
+     aud_set_int ("alarm", day_flags[daynum], alarm_conf.day[daynum].flags);
+     aud_set_int ("alarm", day_h[daynum], alarm_conf.day[daynum].hour);
+     aud_set_int ("alarm", day_m[daynum], alarm_conf.day[daynum].min);
    }
 
    /* END: days of week */
 
-   volume =
-     gtk_range_get_adjustment(alarm_conf.volume)->value;
-   aud_cfg_db_set_int(conf, "alarm", "volume", volume);
+   volume = gtk_range_get_value (alarm_conf.volume);
+   aud_set_int ("alarm", "volume", volume);
 
-   quietvol =
-     gtk_range_get_adjustment(alarm_conf.quietvol)->value;
-   aud_cfg_db_set_int(conf, "alarm", "quietvol", quietvol);
+   quietvol = gtk_range_get_value (alarm_conf.quietvol);
+   aud_set_int ("alarm", "quietvol", quietvol);
 
-   fading =
-     gtk_spin_button_get_value_as_int(alarm_conf.fading);
-   //xmms_cfg_write_int(conf, "alarm", "fading", fading);
+   fading = gtk_spin_button_get_value_as_int (alarm_conf.fading);
 
    /* lets check to see if we need to show the bug warning */
    if((stop_on == TRUE) &&
       ((((stop_h * 60) + stop_m) * 60) < (fading + 65)))
    {
-	   DEBUG("Displaying bug warning, stop %dh %dm, fade %d\n",
-	         stop_h, stop_m, fading);
-	   alarm_warning();
+       AUDDBG("Displaying bug warning, stop %dh %dm, fade %d\n",
+             stop_h, stop_m, fading);
+       alarm_warning();
    }
    else if((stop_on == TRUE) && (fading < 10))
    {
-     DEBUG("Displaying bug warning, stop %dh %dm, fade %d\n",
+     AUDDBG("Displaying bug warning, stop %dh %dm, fade %d\n",
            stop_h, stop_m, fading);
      alarm_warning();
    }
    else
    {
-	   /* write the new values */
-	   aud_cfg_db_set_int(conf, "alarm", "stop_h", stop_h);
-	   aud_cfg_db_set_int(conf, "alarm", "stop_m", stop_m);
-	   aud_cfg_db_set_int(conf, "alarm", "fading", fading);
-	   aud_cfg_db_set_bool(conf, "alarm", "stop_on", stop_on);
+       /* write the new values */
+       aud_set_int ("alarm", "stop_h", stop_h);
+       aud_set_int ("alarm", "stop_m", stop_m);
+       aud_set_int ("alarm", "fading", fading);
+       aud_set_bool ("alarm", "stop_on", stop_on);
    }
 
+   g_free (cmdstr);
+   cmdstr = gtk_editable_get_chars ((GtkEditable *) alarm_conf.cmdstr, 0, -1);
+   aud_set_string ("alarm", "cmdstr", cmdstr);
 
-   g_free(cmdstr);
-   cmdstr = gtk_editable_get_chars(GTK_EDITABLE(alarm_conf.cmdstr),
-				   0, -1);
-   aud_cfg_db_set_string(conf, "alarm", "cmdstr", cmdstr);
+   cmd_on = gtk_toggle_button_get_active (alarm_conf.cmd_on);
+   aud_set_bool ("alarm", "cmd_on", cmd_on);
 
-   cmd_on =
-     gtk_toggle_button_get_active(alarm_conf.cmd_on);
-   aud_cfg_db_set_bool(conf, "alarm", "cmd_on", cmd_on);
-
-   g_free(playlist);
-   playlist = gtk_editable_get_chars(GTK_EDITABLE(alarm_conf.playlist),
-				     0, -1);
-   aud_cfg_db_set_string(conf, "alarm", "playlist", playlist);
+   g_free (playlist);
+   playlist = gtk_editable_get_chars ((GtkEditable *) alarm_conf.playlist, 0, -1);
+   aud_set_string ("alarm", "playlist", playlist);
 
    /* reminder */
-   g_free(alarm_conf.reminder_msg);
-   alarm_conf.reminder_msg = gtk_editable_get_chars(GTK_EDITABLE(alarm_conf.reminder),
-       0, -1);
-   aud_cfg_db_set_string(conf, "alarm", "reminder_msg", alarm_conf.reminder_msg);
-   
-   alarm_conf.reminder_on = 
-     gtk_toggle_button_get_active(alarm_conf.reminder_cb);
-   aud_cfg_db_set_bool(conf, "alarm", "reminder_on", alarm_conf.reminder_on);
+   g_free (alarm_conf.reminder_msg);
+   alarm_conf.reminder_msg = gtk_editable_get_chars ((GtkEditable *) alarm_conf.reminder, 0, -1);
+   aud_set_string ("alarm", "reminder_msg", alarm_conf.reminder_msg);
 
-   aud_cfg_db_close(conf);
+   alarm_conf.reminder_on = gtk_toggle_button_get_active (alarm_conf.reminder_cb);
+   aud_set_bool ("alarm", "reminder_on", alarm_conf.reminder_on);
 }
 
 /*
  * read the current configuration from the file
  */
-static void alarm_read_config()
+static void alarm_read_config(void)
 {
    int daynum = 0;   // used for day number
-   mcs_handle_t *conf;
 
-   DEBUG("alarm_read_config\n");
+   aud_config_set_defaults ("alarm", alarm_defaults);
 
-   conf = aud_cfg_db_open();
-
-   if(!aud_cfg_db_get_int(conf, "alarm", "alarm_h", &alarm_h))
-     alarm_h = DEFAULT_ALARM_HOUR;
-   if(!aud_cfg_db_get_int(conf, "alarm", "alarm_m", &alarm_m))
-     alarm_m = DEFAULT_ALARM_MIN;
+   alarm_h = aud_get_int ("alarm", "alarm_h");
+   alarm_m = aud_get_int ("alarm", "alarm_m");
 
    /* save them here too */
    alarm_conf.default_hour = alarm_h;
    alarm_conf.default_min = alarm_m;
 
-   if(!aud_cfg_db_get_int( conf, "alarm", "stop_h", &stop_h))
-     stop_h = DEFAULT_STOP_HOURS;
-   if(!aud_cfg_db_get_int( conf, "alarm", "stop_m", &stop_m))
-     stop_m = DEFAULT_STOP_MINS;
-   if(!aud_cfg_db_get_bool(conf, "alarm", "stop_on", &stop_on))
-     stop_on = TRUE;
+   stop_h = aud_get_int ("alarm", "stop_h");
+   stop_m = aud_get_int ("alarm", "stop_m");
+   stop_on = aud_get_bool ("alarm", "stop_on");
 
-   if(!aud_cfg_db_get_int(conf, "alarm", "volume", &volume))
-     volume = DEFAULT_VOLUME;
-   if(!aud_cfg_db_get_int(conf, "alarm", "quietvol", &quietvol))
-     quietvol = DEFAULT_QUIET_VOL;
+   volume = aud_get_int ("alarm", "volume");
+   quietvol = aud_get_int ("alarm", "quietvol");
 
-   if(!aud_cfg_db_get_int(conf, "alarm", "fading", &fading))
-     fading = DEFAULT_FADING;
+   fading = aud_get_int ("alarm", "fading");
 
-   if(!aud_cfg_db_get_string(conf, "alarm", "cmdstr", &cmdstr))
-     cmdstr = g_strdup("");
-   if(!aud_cfg_db_get_bool(conf, "alarm", "cmd_on", &cmd_on))
-     cmd_on = FALSE;
+   g_free (cmdstr);
+   cmdstr = aud_get_string ("alarm", "cmdstr");
+   cmd_on = aud_get_bool ("alarm", "cmd_on");
 
-   if(!aud_cfg_db_get_string(conf, "alarm", "playlist", &playlist))
-     playlist = g_strdup("");
+   g_free (playlist);
+   playlist = aud_get_string ("alarm", "playlist");
 
-   if(!aud_cfg_db_get_string(conf, "alarm", "reminder_msg", &alarm_conf.reminder_msg))
-     alarm_conf.reminder_msg = g_strdup("");
-   if(!aud_cfg_db_get_bool(conf, "alarm", "reminder_on", &alarm_conf.reminder_on))
-     alarm_conf.reminder_on = FALSE;
+   g_free (alarm_conf.reminder_msg);
+   alarm_conf.reminder_msg = aud_get_string ("alarm", "reminder_msg");
+   alarm_conf.reminder_on = aud_get_bool ("alarm", "reminder_on");
 
    /* day flags and times */
    for(; daynum < 7; daynum++)
    {
      /* read the flags */
-     if(!aud_cfg_db_get_int(conf, "alarm", day_flags[daynum], &alarm_conf.day[daynum].flags)) {
-       // only turn alarm off by default on a sunday
-       if(daynum != 0)
-         alarm_conf.day[daynum].flags = DEFAULT_FLAGS;
-       else
-         alarm_conf.day[daynum].flags = DEFAULT_FLAGS | ALARM_OFF;
-     }
+     alarm_conf.day[daynum].flags = aud_get_int ("alarm", day_flags[daynum]);
 
      /* read the times */
-     if(!aud_cfg_db_get_int(conf, "alarm", day_h[daynum], &alarm_conf.day[daynum].hour))
-       alarm_conf.day[daynum].hour = DEFAULT_ALARM_HOUR;
-
-     if(!aud_cfg_db_get_int(conf, "alarm", day_m[daynum], &alarm_conf.day[daynum].min))
-       alarm_conf.day[daynum].min = DEFAULT_ALARM_MIN;
+     alarm_conf.day[daynum].hour = aud_get_int ("alarm", day_h[daynum]);
+     alarm_conf.day[daynum].min = aud_get_int ("alarm", day_m[daynum]);
    }
-
-   DEBUG("END alarm_read_config\n");
 }
 
 /*
  * display an about box
  */
-static void alarm_about()
+static void alarm_about(void)
 {
    static GtkWidget *about_dialog = NULL;
 
-   DEBUG("alarm_about\n");
-
-   if(dialog_visible(about_dialog))
+   if (about_dialog)
      return;
 
    about_dialog = create_about_dialog();
 
-   gtk_signal_connect(GTK_OBJECT(about_dialog), "destroy",
-		      GTK_SIGNAL_FUNC(dialog_destroyed), &about_dialog);
+   g_signal_connect (about_dialog, "destroy", (GCallback) gtk_widget_destroyed,
+    & about_dialog);
 
    gtk_widget_show_all(about_dialog);
 
    return;
 }
 
-/*
- * create a playlist file selection dialog
- */
-static void alarm_playlist_browse(GtkButton *button, gpointer data)
-{
-   GtkWidget *fs;
-   gchar *dirname, *path;
-
-   dirname = g_dirname(playlist);
-   DEBUG("dirname = %s\n", dirname);
-   path = g_strdup_printf("%s/", dirname);
-   DEBUG("path = %s\n", path);
-   g_free(dirname);
-
-   fs = create_playlist_fileselection();
-
-   gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), path);
-   g_free(path);
-
-   gtk_widget_show_all(fs);
-}
-
-/*
- * save selected playlist to the corresponding text entry
- */
-void alarm_store_playlistname(GtkButton *button, gpointer data)
-{
-   GtkFileSelection *fs = GTK_FILE_SELECTION(data);
-   gchar *plist;
-
-   DEBUG("alarm_store_playlistname\n");
-
-   plist = g_strdup(gtk_file_selection_get_filename(fs));
-
-   gtk_entry_set_text(alarm_conf.playlist, plist);
-   g_free(plist);
-}
-
 /*
  * displays the configuration window and opens the config file.
  */
@@ -425,12 +365,7 @@ static void alarm_configure(void)
    int daynum = 0;  // used to loop days
    GtkWidget *w;
 
-   DEBUG("alarm_configure\n");
-
-   /*
-    * dont want to show more than one config window
-    */
-   if(dialog_visible(config_dialog))
+   if (config_dialog)
      return;
 
    alarm_read_config();
@@ -463,18 +398,18 @@ static void alarm_configure(void)
    w = lookup_widget(config_dialog, "vol_scale");
    alarm_conf.volume = GTK_RANGE(w);
    gtk_range_set_adjustment(alarm_conf.volume,
-			    GTK_ADJUSTMENT(gtk_adjustment_new(volume,
-							      0,
-							      100, 1,
-							      5, 0)));
+                GTK_ADJUSTMENT(gtk_adjustment_new(volume,
+                                  0,
+                                  100, 1,
+                                  5, 0)));
 
    w = lookup_widget(config_dialog, "quiet_vol_scale");
    alarm_conf.quietvol = GTK_RANGE(w);
    gtk_range_set_adjustment(alarm_conf.quietvol,
-			    GTK_ADJUSTMENT(gtk_adjustment_new(quietvol,
-							      0,
-							      100, 1,
-							      5, 0)));
+                GTK_ADJUSTMENT(gtk_adjustment_new(quietvol,
+                                  0,
+                                  100, 1,
+                                  5, 0)));
 
    /* days of week */
    for(; daynum < 7; daynum++)
@@ -482,14 +417,14 @@ static void alarm_configure(void)
      w = lookup_widget(config_dialog, day_cb[daynum]);
      alarm_conf.day[daynum].cb = GTK_CHECK_BUTTON(w);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alarm_conf.day[daynum].cb),
-				!(alarm_conf.day[daynum].flags & ALARM_OFF));
+                !(alarm_conf.day[daynum].flags & ALARM_OFF));
 
      w = lookup_widget(config_dialog, day_def[daynum]);
      alarm_conf.day[daynum].cb_def = GTK_CHECK_BUTTON(w);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alarm_conf.day[daynum].cb_def),
          alarm_conf.day[daynum].flags & ALARM_DEFAULT);
 
-   
+
      /* Changed to show default time instead of set time when ALARM_DEFAULT set,
       * as suggested by Mark Brown
       */
@@ -555,16 +490,12 @@ static void alarm_configure(void)
    alarm_conf.reminder_cb = GTK_TOGGLE_BUTTON(w);
    gtk_toggle_button_set_active(alarm_conf.reminder_cb, alarm_conf.reminder_on);
 
-   w = lookup_widget(config_dialog, "playlist_browse_button");
-   gtk_signal_connect(GTK_OBJECT(w), "clicked",
-		      GTK_SIGNAL_FUNC(alarm_playlist_browse), NULL);
-
-   gtk_signal_connect(GTK_OBJECT(config_dialog), "destroy",
-		      GTK_SIGNAL_FUNC(dialog_destroyed), &config_dialog);
+   g_signal_connect (config_dialog, "destroy", (GCallback) gtk_widget_destroyed,
+    & config_dialog);
 
    gtk_widget_show_all(config_dialog);
 
-   DEBUG("END alarm_configure\n");
+   AUDDBG("END alarm_configure\n");
 }
 
 /* functions for greying out the time for days */
@@ -643,9 +574,9 @@ void alarm_current_volume(GtkButton *button, gpointer data)
    gint vol;
    GtkAdjustment *adj;
 
-   DEBUG("on_current_button_clicked\n");
+   AUDDBG("on_current_button_clicked\n");
 
-   audacious_drct_get_volume_main(&vol);
+   aud_drct_get_volume_main(&vol);
 
    adj = gtk_range_get_adjustment(alarm_conf.volume);
    gtk_adjustment_set_value(adj, (gfloat)vol);
@@ -657,30 +588,30 @@ void alarm_current_volume(GtkButton *button, gpointer data)
  */
 static void threadsleep(float x)
 {
-   DEBUG("threadsleep: waiting %f seconds\n", x);
+   AUDDBG("threadsleep: waiting %f seconds\n", x);
 
    g_usleep((int) ((float) x * (float) 1000000.0));
 
    return;
 }
 
-static inline pthread_t alarm_thread_create(void *(*start_routine)(void *), void *args, unsigned int detach)
+static inline alarm_thread_t alarm_thread_create(void *(*start_routine)(void *), void *args, unsigned int detach)
 {
-   pthread_t tid;
+   alarm_thread_t thrd;
    pthread_attr_t attr;
 
    pthread_attr_init(&attr);
 
    if(detach != 0)
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-    
+
    pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED);
    pthread_attr_setschedpolicy(&attr, SCHED_OTHER);
    pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
 
-   pthread_create(&tid, &attr, start_routine, args);
+   thrd.is_valid = (pthread_create(&thrd.tid, &attr, start_routine, args) == 0);
 
-   return(tid);
+   return thrd;
 }
 
 static void *alarm_fade(void *arg)
@@ -694,7 +625,7 @@ static void *alarm_fade(void *arg)
    pthread_mutex_lock(&fader_lock);
 
    /* slide volume */
-   /* the Kaspar Giger way of fading, check the current mixer volume and 
+   /* the Kaspar Giger way of fading, check the current mixer volume and
     * increment from there so that if you have some other app lowering the
     * volume at the same time xmms-alarm will not ignore it.  If you have some
     * other app increasing the volume, then it could get louder that you expect
@@ -710,25 +641,25 @@ static void *alarm_fade(void *arg)
      inc = -1;
    else
      inc = 1;
-   
-   audacious_drct_set_volume_main((gint)vols->start);
+
+   aud_drct_set_volume_main((gint)vols->start);
    //for(i=0;i<(vols->end - vols->start);i++)
    for(i=0;i<adiff;i++)
    {
      //threadsleep((gfloat)fading / (vols->end - vols->start));
      threadsleep((gfloat)fading / (gfloat)adiff);
-     audacious_drct_get_volume_main(&v);
-     audacious_drct_set_volume_main(v + inc);
+     aud_drct_get_volume_main(&v);
+     aud_drct_set_volume_main(v + inc);
    }
    /* Setting the volume to the end volume sort of defeats the point if having
     * the code in there to allow other apps to control volume too :)
     */
-   //audacious_drct_set_volume_main((gint)vols->end);
+   //aud_drct_set_volume_main((gint)vols->end);
 
    /* and */
    pthread_mutex_unlock(&fader_lock);
 
-   DEBUG("volume = %f%%\n", (gdouble)vols->end);
+   AUDDBG("volume = %f%%\n", (gdouble)vols->end);
    return(0);
 }
 
@@ -736,9 +667,9 @@ static void *alarm_stop_thread( void *args )
 {
    gint currvol;
    fader fade_vols;
-   pthread_t f_tid;
+   alarm_thread_t f;
 
-   DEBUG("alarm_stop_thread\n");
+   AUDDBG("alarm_stop_thread\n");
 
 
    /* sleep for however long we are meant to be sleeping for until
@@ -746,37 +677,39 @@ static void *alarm_stop_thread( void *args )
     */
    threadsleep(((stop_h * 60) + stop_m) * 60);
 
-   DEBUG("alarm_stop triggered\n");
+   AUDDBG("alarm_stop triggered\n");
 
-   if (dialog_visible(alarm_dialog))
+   if (alarm_dialog)
      gtk_widget_destroy(alarm_dialog);
 
-   audacious_drct_get_volume_main(&currvol),
+   aud_drct_get_volume_main(&currvol),
 
    /* fade back to zero */
    fade_vols.start = currvol;
    fade_vols.end = 0;
 
    /* The fader thread locks the fader_mutex now */
-   f_tid = alarm_thread_create(alarm_fade, &fade_vols, 0);
+   f = alarm_thread_create(alarm_fade, &fade_vols, 0);
 
-   pthread_join(f_tid, NULL);
-   audacious_drct_stop();
+   pthread_join(f.tid, NULL);
+   aud_drct_stop();
 
    /* might as well set the volume to something higher than zero so we
     * dont confuse the poor people who just woke up and cant work out why
     * theres no music playing when they press the little play button :)
     */
-   audacious_drct_set_volume_main(currvol);
+   aud_drct_set_volume_main(currvol);
 
-   DEBUG("alarm_stop done\n");
+   AUDDBG("alarm_stop done\n");
    return(NULL);
 }
 
 void alarm_stop_cancel(GtkButton *w, gpointer data)
 {
-   DEBUG("alarm_stop_cancel\n");
-   pthread_cancel(stop_tid);
+   AUDDBG("alarm_stop_cancel\n");
+   if (pthread_cancel(stop.tid) == 0) {
+      stop.is_valid = FALSE;
+   }
 }
 
 /* the main alarm thread */
@@ -790,26 +723,26 @@ static void *alarm_start_thread(void *args)
    /* give it time to set start_tid to something */
    threadsleep(1);
 
-   while(start_tid != 0)
+   while (start.is_valid)
    {
      /* sit around and wait for the faders to not be doing anything */
-     DEBUG("Waiting for fader to be unlocked..");
+     AUDDBG("Waiting for fader to be unlocked..");
      pthread_mutex_lock(&fader_lock);
-     DEBUG("Ok\n");
+     AUDDBG("Ok\n");
      pthread_mutex_unlock(&fader_lock);
 
-     DEBUG("Getting time\n");
+     AUDDBG("Getting time\n");
      timenow = time(NULL);
      currtime = localtime(&timenow);
      today = currtime->tm_wday;
-     DEBUG("Today is %d\n", today);
+     AUDDBG("Today is %d\n", today);
 
      /* see if its time to do something */
-     DEBUG("Checking Day\n");
+     AUDDBG("Checking Day\n");
 
      /* Had to put something here so I put the hour string.
      ** Its only debug stuff anyway */
-     DEBUG(day_h[today]);
+     AUDDBG("%s",day_h[today]);
 
      if(alarm_conf.day[today].flags & ALARM_OFF)
      {
@@ -831,10 +764,10 @@ static void *alarm_start_thread(void *args)
        }
      }
 
-     DEBUG("Alarm time is %d:%d (def: %d:%d)\n", alarm_h, alarm_m,
+     AUDDBG("Alarm time is %d:%d (def: %d:%d)\n", alarm_h, alarm_m,
         alarm_conf.default_hour, alarm_conf.default_min);
 
-     DEBUG("Checking time (%d:%d)\n", currtime->tm_hour, currtime->tm_min);
+     AUDDBG("Checking time (%d:%d)\n", currtime->tm_hour, currtime->tm_min);
      if((currtime->tm_hour != alarm_h) || (currtime->tm_min != alarm_m))
      {
        threadsleep(8.5);
@@ -843,36 +776,36 @@ static void *alarm_start_thread(void *args)
 
      if(cmd_on == TRUE)
      {
-       DEBUG("Executing %s, cmd_on is true\n", cmdstr);
+       AUDDBG("Executing %s, cmd_on is true\n", cmdstr);
        if(system(cmdstr) == -1)
        {
-        DEBUG("Executing %s failed\n",cmdstr);
+        AUDDBG("Executing %s failed\n",cmdstr);
        }
      }
 
-     DEBUG("strcmp playlist, playlist is [%s]\n", playlist);
+     AUDDBG("strcmp playlist, playlist is [%s]\n", playlist);
      if(strcmp(playlist, ""))
      {
-       DEBUG("playlist is not blank, aparently\n");
+       AUDDBG("playlist is not blank, aparently\n");
        GList list;
 
        list.prev = list.next = NULL;
        list.data = playlist;
 
-       audacious_drct_pl_clear();
-       audacious_drct_pl_add(&list);
+       aud_drct_pl_clear();
+       aud_drct_pl_add_list (& list, -1);
      }
 
      if(fading)
      {
        fader fade_vols;
-       
-       DEBUG("Fading is true\n");
-       audacious_drct_set_volume_main(quietvol);
+
+       AUDDBG("Fading is true\n");
+       aud_drct_set_volume_main(quietvol);
 
        /* start playing */
        play_start = time(NULL);
-       audacious_drct_play();
+       aud_drct_play();
 
        /* fade volume */
        fade_vols.start = quietvol;
@@ -886,24 +819,22 @@ static void *alarm_start_thread(void *args)
        /* no fading */
 
        /* set volume */
-       audacious_drct_set_volume_main(volume);
+       aud_drct_set_volume_main(volume);
 
        /* start playing */
        play_start = time(NULL);
-       audacious_drct_play();
+       aud_drct_play();
      }
 
      if(alarm_conf.reminder_on == TRUE)
      {
        GtkWidget *reminder_dialog;
-       DEBUG("Showing reminder '%s'\n", alarm_conf.reminder_msg);
+       AUDDBG("Showing reminder '%s'\n", alarm_conf.reminder_msg);
 
-       GDK_THREADS_ENTER();
        reminder_dialog = (GtkWidget*) create_reminder_dialog(alarm_conf.reminder_msg);
-       gtk_signal_connect(GTK_OBJECT(reminder_dialog), "destroy",
-       GTK_SIGNAL_FUNC(dialog_destroyed), &reminder_dialog);
+       g_signal_connect (reminder_dialog, "destroy", (GCallback)
+        gtk_widget_destroyed, & reminder_dialog);
        gtk_widget_show_all(reminder_dialog);
-       GDK_THREADS_LEAVE();
      }
 
      /* bring up the wakeup call dialog if stop_on is set TRUE, this
@@ -915,39 +846,28 @@ static void *alarm_start_thread(void *args)
       */
       if(stop_on == TRUE)
       {
-         /* ok, so when we want to open dialogs in threaded programs
-          * we use this do we?
-          * anyone?
-          */
-          GDK_THREADS_ENTER();
-          {
-            DEBUG("stop_on is true\n");
             alarm_dialog = create_alarm_dialog();
-            DEBUG("created alarm dialog, %p\n", alarm_dialog);
+            g_signal_connect (alarm_dialog, "destroy", (GCallback)
+             gtk_widget_destroyed, & alarm_dialog);
 
-            gtk_signal_connect(GTK_OBJECT(alarm_dialog), "destroy",
-            GTK_SIGNAL_FUNC(dialog_destroyed), &alarm_dialog);
-            DEBUG("attached destroy signal to alarm dialog, %p\n", alarm_dialog);
             gtk_widget_show_all(alarm_dialog);
-            DEBUG("dialog now showing\n");
 
-            DEBUG("now starting stop thread\n");
-            stop_tid = alarm_thread_create(alarm_stop_thread, NULL, 0);
-            DEBUG("Created wakeup dialog and started stop thread(%d)\n", (int)stop_tid);
-
-	        }
-	        GDK_THREADS_LEAVE();
+            AUDDBG("now starting stop thread\n");
+            stop = alarm_thread_create(alarm_stop_thread, NULL, 0);
+            AUDDBG("Created wakeup dialog and started stop thread\n");
 
           /* now wait for the stop thread */
-          DEBUG("Waiting for stop to stop.... (%d)", (int)stop_tid);
-          pthread_join(stop_tid, NULL);
+          AUDDBG("Waiting for stop to stop.... ");
+          if (pthread_join(stop.tid, NULL) == 0)
+             stop.is_valid = FALSE;
+
           /* loop until we are out of the starting minute */
           while(time(NULL) < (play_start + 61))
           {
-            DEBUG("Waiting until out of starting minute\n");
+            AUDDBG("Waiting until out of starting minute\n");
             threadsleep(5.0);
           }
-          DEBUG("OK\n");
+          AUDDBG("OK\n");
       }
       /* loop until we are out of the starting minute */
       while(time(NULL) < (play_start + 61))
@@ -957,7 +877,7 @@ static void *alarm_start_thread(void *args)
       threadsleep(fading);
    }
 
-   DEBUG("Main thread has gone...\n");
+   AUDDBG("Main thread has gone...\n");
    return NULL;
 }
 
@@ -966,39 +886,48 @@ static void *alarm_start_thread(void *args)
  * opens the config file and reads the value, creates a new
  * config in memory if the file doesnt exist and sets default vals
  */
-static void alarm_init()
+static gboolean alarm_init (void)
 {
-   DEBUG("alarm_init\n");
+   AUDDBG("alarm_init\n");
 
+   alarm_conf.reminder_msg = NULL;
    alarm_read_config();
 
    /* start the main thread running */
-   start_tid = alarm_thread_create(alarm_start_thread, NULL, 1);
+   start = alarm_thread_create(alarm_start_thread, NULL, 1);
+
+   return TRUE;
 }
 
 /*
  * kill the main thread
  */
-static void alarm_cleanup()
+static void alarm_cleanup(void)
 {
-   DEBUG("alarm_cleanup\n");
+   AUDDBG("alarm_cleanup\n");
 
-   if (start_tid)
-     pthread_cancel(start_tid);
-   start_tid = 0;
-   if(stop_tid)
-     pthread_cancel(stop_tid);
-   stop_tid = 0;
+   if (start.is_valid) {
+     pthread_cancel(start.tid);
+     start.is_valid = FALSE;
+   }
+   if (stop.is_valid) {
+     pthread_cancel(stop.tid);
+     stop.is_valid = FALSE;
+   }
+
+   g_free(alarm_conf.reminder_msg);
+   alarm_conf.reminder_msg = NULL;
+   g_free(playlist);
+   playlist = NULL;
+   g_free(cmdstr);
+   cmdstr = NULL;
 }
 
-static GeneralPlugin alarm_plugin =
-{
-     .description = "Alarm "VERSION,
+AUD_GENERAL_PLUGIN
+(
+     .name = "Alarm",
      .init = alarm_init,
      .about = alarm_about,
      .configure = alarm_configure,
      .cleanup = alarm_cleanup,
-};
-
-GeneralPlugin *alarm_gplist[] = { &alarm_plugin, NULL };
-SIMPLE_GENERAL_PLUGIN(alarm, alarm_gplist);
+)
