@@ -1,6 +1,6 @@
 /*
  * ALSA Output Plugin for Audacious
- * Copyright 2009-2010 John Lindgren
+ * Copyright 2009-2011 John Lindgren
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -48,8 +48,9 @@
 #include <alsa/asoundlib.h>
 #include <glib.h>
 
-#include <audacious/audconfig.h>
 #include <audacious/debug.h>
+#include <audacious/misc.h>
+#include <audacious/plugin.h>
 
 #include "alsa.h"
 
@@ -78,10 +79,11 @@ static int alsa_channels, alsa_rate;
 
 static void * alsa_buffer;
 static int alsa_buffer_length, alsa_buffer_data_start, alsa_buffer_data_length;
+static int alsa_period; /* milliseconds */
 
 static int64_t alsa_written; /* frames */
 static char alsa_prebuffer, alsa_paused;
-static int alsa_paused_time; /* milliseconds */
+static int alsa_paused_delay; /* frames */
 
 static int poll_pipe[2];
 static int poll_count;
@@ -138,7 +140,8 @@ static void poll_sleep (void)
 static void poll_wake (void)
 {
     const char c = 0;
-    write (poll_pipe[1], & c, 1);
+    if (write (poll_pipe[1], & c, 1) < 0)
+        ERROR ("Failed to write to pipe: %s.\n", strerror (errno));
 }
 
 static void poll_cleanup (void)
@@ -153,9 +156,14 @@ static void * pump (void * unused)
     pthread_mutex_lock (& alsa_mutex);
     pthread_cond_broadcast (& alsa_cond); /* signal thread started */
 
+    char failed = 0;
+    char workaround = 0;
+    int slept = 0;
+
     while (! pump_quit)
     {
-        if (alsa_prebuffer || alsa_paused || ! alsa_buffer_data_length)
+        if (alsa_prebuffer || alsa_paused || ! snd_pcm_bytes_to_frames
+         (alsa_handle, alsa_buffer_data_length))
         {
             pthread_cond_wait (& alsa_cond, & alsa_mutex);
             continue;
@@ -167,6 +175,8 @@ static void * pump (void * unused)
         if (! length)
             goto WAIT;
 
+        slept = 0;
+
         length = snd_pcm_frames_to_bytes (alsa_handle, length);
         length = MIN (length, alsa_buffer_data_length);
         length = MIN (length, alsa_buffer_length - alsa_buffer_data_start);
@@ -176,6 +186,8 @@ static void * pump (void * unused)
         CHECK_VAL_RECOVER (written, snd_pcm_writei, alsa_handle, (char *)
          alsa_buffer + alsa_buffer_data_start, length);
 
+        failed = 0;
+
         written = snd_pcm_frames_to_bytes (alsa_handle, written);
         alsa_buffer_data_start += written;
         alsa_buffer_data_length -= written;
@@ -188,13 +200,41 @@ static void * pump (void * unused)
             continue;
         }
 
+        if (! snd_pcm_bytes_to_frames (alsa_handle, alsa_buffer_data_length))
+            continue;
+
     WAIT:
         pthread_mutex_unlock (& alsa_mutex);
-        poll_sleep ();
+
+        if (slept > 4)
+        {
+            AUDDBG ("Activating timer workaround.\n");
+            workaround = 1;
+        }
+
+        if (workaround && slept)
+        {
+            const struct timespec delay = {.tv_sec = 0, .tv_nsec = 600000 *
+             alsa_period};
+            nanosleep (& delay, NULL);
+        }
+        else
+        {
+            poll_sleep ();
+            slept ++;
+        }
+
         pthread_mutex_lock (& alsa_mutex);
+        continue;
+
+    FAILED:
+        if (failed)
+            break;
+
+        failed = 1;
+        CHECK (snd_pcm_prepare, alsa_handle);
     }
 
-FAILED:
     pthread_mutex_unlock (& alsa_mutex);
     return NULL;
 }
@@ -238,17 +278,11 @@ FAILED:
     return delay;
 }
 
-static int get_output_time (void)
-{
-    return (int64_t) (alsa_written - snd_pcm_bytes_to_frames (alsa_handle,
-     alsa_buffer_data_length) - get_delay ()) * 1000 / alsa_rate;
-}
-
-OutputPluginInitStatus alsa_init (void)
+int alsa_init (void)
 {
     alsa_handle = NULL;
     alsa_initted = 0;
-    return OUTPUT_PLUGIN_INIT_FOUND_DEVICES;
+    return 1;
 }
 
 void alsa_soft_init (void)
@@ -337,7 +371,8 @@ int alsa_open_audio (int aud_format, int rate, int channels)
     alsa_channels = channels;
     alsa_rate = rate;
 
-    unsigned int useconds = 1000 * MIN (1000, aud_cfg->output_buffer_size / 2);
+    int total_buffer = aud_get_int (NULL, "output_buffer_size");
+    unsigned int useconds = 1000 * MIN (1000, total_buffer / 2);
     int direction = 0;
     CHECK_NOISY (snd_pcm_hw_params_set_buffer_time_near, alsa_handle, params,
      & useconds, & direction);
@@ -347,16 +382,13 @@ int alsa_open_audio (int aud_format, int rate, int channels)
     direction = 0;
     CHECK_NOISY (snd_pcm_hw_params_set_period_time_near, alsa_handle, params,
      & useconds, & direction);
-#ifdef DEBUG
-    int period = useconds / 1000;
-#endif
+    alsa_period = useconds / 1000;
 
     CHECK_NOISY (snd_pcm_hw_params, alsa_handle, params);
 
-    int soft_buffer = MAX (aud_cfg->output_buffer_size / 2,
-     aud_cfg->output_buffer_size - hard_buffer);
+    int soft_buffer = MAX (total_buffer / 2, total_buffer - hard_buffer);
     AUDDBG ("Buffer: hardware %d ms, software %d ms, period %d ms.\n",
-     hard_buffer, soft_buffer, period);
+     hard_buffer, soft_buffer, alsa_period);
 
     alsa_buffer_length = snd_pcm_frames_to_bytes (alsa_handle, (int64_t)
      soft_buffer * rate / 1000);
@@ -367,7 +399,7 @@ int alsa_open_audio (int aud_format, int rate, int channels)
     alsa_written = 0;
     alsa_prebuffer = 1;
     alsa_paused = 0;
-    alsa_paused_time = 0;
+    alsa_paused_delay = 0;
 
     if (! poll_setup ())
         goto FAILED;
@@ -437,6 +469,9 @@ void alsa_write_audio (void * data, int length)
     alsa_buffer_data_length += length;
     alsa_written += snd_pcm_bytes_to_frames (alsa_handle, length);
 
+    if (! alsa_paused)
+        pthread_cond_broadcast (& alsa_cond);
+
     pthread_mutex_unlock (& alsa_mutex);
 }
 
@@ -470,7 +505,7 @@ void alsa_drain (void)
     if (alsa_prebuffer)
         start_playback ();
 
-    while (alsa_buffer_data_length > 0)
+    while (snd_pcm_bytes_to_frames (alsa_handle, alsa_buffer_data_length))
         pthread_cond_wait (& alsa_cond, & alsa_mutex);
 
     pump_stop ();
@@ -528,8 +563,17 @@ int alsa_written_time (void)
 int alsa_output_time (void)
 {
     pthread_mutex_lock (& alsa_mutex);
-    int time = (alsa_prebuffer || alsa_paused) ? alsa_paused_time :
-     get_output_time ();
+
+    int64_t frames = alsa_written - snd_pcm_bytes_to_frames (alsa_handle,
+     alsa_buffer_data_length);
+
+    if (alsa_prebuffer || alsa_paused)
+        frames -= alsa_paused_delay;
+    else
+        frames -= get_delay ();
+
+    int time = frames * 1000 / alsa_rate;
+
     pthread_mutex_unlock (& alsa_mutex);
     return time;
 }
@@ -548,7 +592,7 @@ FAILED:
 
     alsa_written = (int64_t) time * alsa_rate / 1000;
     alsa_prebuffer = 1;
-    alsa_paused_time = time;
+    alsa_paused_delay = 0;
 
     pthread_cond_broadcast (& alsa_cond); /* interrupt period wait */
 
@@ -557,7 +601,7 @@ FAILED:
     pthread_mutex_unlock (& alsa_mutex);
 }
 
-void alsa_pause (short pause)
+void alsa_pause (int pause)
 {
     AUDDBG ("%sause.\n", pause ? "P" : "Unp");
     pthread_mutex_lock (& alsa_mutex);
@@ -567,7 +611,7 @@ void alsa_pause (short pause)
     if (! alsa_prebuffer)
     {
         if (pause)
-            alsa_paused_time = get_output_time ();
+            alsa_paused_delay = get_delay ();
 
         CHECK (snd_pcm_pause, alsa_handle, pause);
     }
