@@ -85,13 +85,25 @@ aosd_callback_list_free ( GList * list )
 
 static gboolean
 aosd_cb_configure_position_expose ( GtkWidget * darea ,
+#if GTK_CHECK_VERSION (3, 0, 0)
+                                    cairo_t * cr ,
+#else
                                     GdkEventExpose * event ,
+#endif
                                     gpointer coord_gp )
 {
   gint coord = GPOINTER_TO_INT(coord_gp);
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+  cairo_set_source_rgb ( cr , 0 , 0 , 0 );
+  cairo_rectangle ( cr , (coord % 3) * 10 , (coord / 3) * 16 , 20 , 8 );
+  cairo_fill ( cr );
+#else
   gdk_draw_rectangle( GDK_DRAWABLE(darea->window) ,
                       darea->style->black_gc , TRUE ,
                       (coord % 3) * 10 , (coord / 3) * 16 , 20 , 8 );
+#endif
+
   return FALSE;
 }
 
@@ -182,8 +194,13 @@ aosd_ui_configure_position ( aosd_cfg_t * cfg , GList ** cb_list )
     pos_placement_bt_darea[i] = gtk_drawing_area_new();
     gtk_widget_set_size_request( pos_placement_bt_darea[i] , 40 , 40 );
     gtk_container_add( GTK_CONTAINER(pos_placement_bt[i]) , pos_placement_bt_darea[i] );
+#if GTK_CHECK_VERSION (3, 0, 0)
+    g_signal_connect( G_OBJECT(pos_placement_bt_darea[i]) , "draw" ,
+                      G_CALLBACK(aosd_cb_configure_position_expose) , GINT_TO_POINTER(i) );
+#else
     g_signal_connect( G_OBJECT(pos_placement_bt_darea[i]) , "expose-event" ,
                       G_CALLBACK(aosd_cb_configure_position_expose) , GINT_TO_POINTER(i) );
+#endif
     gtk_table_attach( GTK_TABLE(pos_placement_table) , pos_placement_bt[i] ,
                       (i % 3) , (i % 3) + 1 , (i / 3) , (i / 3) + 1 ,
                       GTK_FILL , GTK_FILL , 0 , 0 );
@@ -235,14 +252,25 @@ aosd_ui_configure_position ( aosd_cfg_t * cfg , GList ** cb_list )
   gtk_container_set_border_width( GTK_CONTAINER(pos_multimon_hbox) , 6 );
   gtk_container_add( GTK_CONTAINER(pos_multimon_frame), pos_multimon_hbox );
   pos_multimon_label = gtk_label_new( _("Display OSD using:") );
-  pos_multimon_combobox = gtk_combo_box_new_text();
-  gtk_combo_box_append_text( GTK_COMBO_BOX(pos_multimon_combobox) , _("all monitors") );
+#if GTK_CHECK_VERSION (3, 0, 0)
+  #define CBT_NEW gtk_combo_box_text_new
+  #define CBT_ADD(widget, text) \
+    gtk_combo_box_text_append( GTK_COMBO_BOX_TEXT(widget) , NULL , text )
+#else
+  #define CBT_NEW gtk_combo_box_new_text
+  #define CBT_ADD(widget, text) \
+    gtk_combo_box_append_text( GTK_COMBO_BOX(widget) , text )
+#endif
+  pos_multimon_combobox = CBT_NEW();
+  CBT_ADD( pos_multimon_combobox , _("all monitors") );
   for ( i = 0 ; i < monitors_num ; i++ )
   {
     gchar *mon_str = g_strdup_printf( _("monitor %i") , i + 1 );
-    gtk_combo_box_append_text( GTK_COMBO_BOX(pos_multimon_combobox) , mon_str );
+    CBT_ADD( pos_multimon_combobox , mon_str );
     g_free( mon_str );
   }
+#undef CBT_NEW
+#undef CBT_ADD
   gtk_combo_box_set_active( GTK_COMBO_BOX(pos_multimon_combobox) , (cfg->osd->position.multimon_id + 1) );
   aosd_callback_list_add( cb_list , pos_multimon_combobox , aosd_cb_configure_position_multimon_commit );
   gtk_box_pack_start( GTK_BOX(pos_multimon_hbox) , pos_multimon_label , FALSE , FALSE , 0 );
@@ -927,9 +955,6 @@ aosd_cb_configure_test ( gpointer cfg_win )
   GList *cb_list = g_object_get_data( G_OBJECT(cfg_win) , "cblist" );
   aosd_callback_list_run( cb_list , cfg );
   cfg->set = TRUE;
-#ifdef DEBUG
-  aosd_cfg_debug( cfg );
-#endif
   markup_message = g_markup_printf_escaped(
     _("<span font_desc='%s'>Audacious OSD</span>") , cfg->osd->text.fonts_name[0] );
   aosd_osd_shutdown(); /* stop any displayed osd */
