@@ -1,4 +1,4 @@
-// Game_Music_Emu 0.5.2. http://www.slack.net/~ant/
+// Game_Music_Emu 0.5.5. http://www.slack.net/~ant/
 
 /*
 Last validated with zexall 2006.11.21 5:26 PM
@@ -805,8 +805,6 @@ possibly_out_of_time:
 //////////////////////////////////////// CB prefix
 	{
 	case 0xCB:
-		unsigned data2;
-		data2 = INSTR( 1 );
 		pc++;
 		switch ( data )
 		{
@@ -1252,7 +1250,7 @@ possibly_out_of_time:
 		
 		case 0x4F: // LD R,A
 			SET_R( rg.a );
-			dprintf( "LD R,A not supported\n" );
+			debug_printf( "LD R,A not supported\n" );
 			warning = true;
 			goto loop;
 		
@@ -1262,7 +1260,7 @@ possibly_out_of_time:
 		
 		case 0x5F: // LD A,R
 			rg.a = GET_R();
-			dprintf( "LD A,R not supported\n" );
+			debug_printf( "LD A,R not supported\n" );
 			warning = true;
 		ld_ai_common:
 			flags = (flags & C01) | SZ28( rg.a ) | (r.iff2 << 2 & V04);
@@ -1285,7 +1283,7 @@ possibly_out_of_time:
 			goto loop;
 		
 		default:
-			dprintf( "Opcode $ED $%02X not supported\n", data );
+			debug_printf( "Opcode $ED $%02X not supported\n", data );
 			warning = true;
 			goto loop;
 		}
@@ -1545,7 +1543,7 @@ possibly_out_of_time:
 			}
 			
 			default:
-				dprintf( "Opcode $%02X $CB $%02X not supported\n", opcode, data2 );
+				debug_printf( "Opcode $%02X $CB $%02X not supported\n", opcode, data2 );
 				warning = true;
 				goto loop;
 			}
@@ -1634,7 +1632,7 @@ possibly_out_of_time:
 		}
 		
 		default:
-			dprintf( "Unnecessary DD/FD prefix encountered\n" );
+			debug_printf( "Unnecessary DD/FD prefix encountered\n" );
 			warning = true;
 			pc--;
 			goto loop;
@@ -1643,7 +1641,7 @@ possibly_out_of_time:
 	}
 	
 	}
-	dprintf( "Unhandled main opcode: $%02X\n", opcode );
+	debug_printf( "Unhandled main opcode: $%02X\n", opcode );
 	assert( false );
 	
 halt:
