@@ -20,7 +20,9 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <audacious/configdb.h>
+#include <gtk/gtk.h>
+
+#include <audacious/gtk-compat.h>
 #include <audacious/misc.h>
 #include <audacious/playlist.h>
 #include <libaudcore/audstrings.h>
@@ -55,7 +57,7 @@ enum fileext_t
     FILEEXT_MAX
 };
 
-static gint fileext = WAV;
+static gint fileext;
 static const gchar *fileext_str[FILEEXT_MAX] =
 {
     "wav",
@@ -73,57 +75,24 @@ static const gchar *fileext_str[FILEEXT_MAX] =
 static FileWriter *plugin;
 
 static GtkWidget *saveplace_hbox, *saveplace;
-static gboolean save_original = TRUE;
+static gboolean save_original;
 
 static GtkWidget *filenamefrom_hbox, *filenamefrom_label, *filenamefrom_toggle;
-static gboolean filenamefromtags = TRUE;
+static gboolean filenamefromtags;
 
 static GtkWidget *use_suffix_toggle = NULL;
-static gboolean use_suffix = FALSE;
+static gboolean use_suffix;
 
 static GtkWidget *prependnumber_toggle;
-static gboolean prependnumber = FALSE;
+static gboolean prependnumber;
 
-static gchar *file_path = NULL;
+static gchar *file_path;
 
 VFSFile *output_file = NULL;
-const Tuple *tuple = NULL;
+Tuple *tuple = NULL;
 
 static gint64 samples_written;
 
-static OutputPluginInitStatus file_init(void);
-static void file_about(void);
-static gint file_open(gint fmt, gint rate, gint nch);
-static void file_write(void *ptr, gint length);
-static gint file_write_output(void *ptr, gint length);
-static void file_close(void);
-static void file_flush(gint time);
-static void file_pause(short p);
-static gint file_playing(void);
-static gint file_get_time (void);
-static void file_configure(void);
-
-OutputPlugin file_op =
-{
-    .description = "FileWriter Plugin",
-    .probe_priority = 0,
-    .init = file_init,
-    .about = file_about,
-    .configure = file_configure,
-    .open_audio = file_open,
-    .write_audio = file_write,
-    .close_audio = file_close,
-    .flush = file_flush,
-    .pause = file_pause,
-    .buffer_playing = file_playing,
-    .output_time = file_get_time,
-    .written_time = file_get_time,
-};
-
-OutputPlugin *file_oplist[] = { &file_op, NULL };
-
-SIMPLE_OUTPUT_PLUGIN (filewriter, file_oplist)
-
 FileWriter *plugins[FILEEXT_MAX] = {
     &wav_plugin,
 #ifdef FILEWRITER_MP3
@@ -145,31 +114,48 @@ static void set_plugin(void)
     plugin = plugins[fileext];
 }
 
-static OutputPluginInitStatus file_init(void)
+static gint file_write_output (void * data, gint length)
 {
-    mcs_handle_t *db;
+    return vfs_fwrite (data, 1, length, output_file);
+}
 
-    db = aud_cfg_db_open();
-    aud_cfg_db_get_int(db, FILEWRITER_CFGID, "fileext", &fileext);
-    aud_cfg_db_get_string(db, FILEWRITER_CFGID, "file_path", &file_path);
-    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "save_original", &save_original);
-    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "use_suffix", &use_suffix);
-    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "filenamefromtags", &filenamefromtags);
-    aud_cfg_db_get_bool(db, FILEWRITER_CFGID, "prependnumber", &prependnumber);
-    aud_cfg_db_close(db);
+static const gchar * const filewriter_defaults[] = {
+ "fileext", "0", /* WAV */
+ "filenamefromtags", "TRUE",
+ "prependnumber", "FALSE",
+ "save_original", "TRUE",
+ "use_suffix", "FALSE",
+ NULL};
 
-    if (file_path == NULL)
+static gboolean file_init (void)
+{
+    aud_config_set_defaults ("filewriter", filewriter_defaults);
+
+    fileext = aud_get_int ("filewriter", "fileext");
+    filenamefromtags = aud_get_bool ("filewriter", "filenamefromtags");
+    file_path = aud_get_string ("filewriter", "file_path");
+    prependnumber = aud_get_bool ("filewriter", "prependnumber");
+    save_original = aud_get_bool ("filewriter", "save_original");
+    use_suffix = aud_get_bool ("filewriter", "use_suffix");
+
+    if (! file_path[0])
     {
-        g_return_val_if_fail (getenv ("HOME") != NULL, OUTPUT_PLUGIN_INIT_FAIL);
+        g_return_val_if_fail (getenv ("HOME") != NULL, FALSE);
         file_path = g_filename_to_uri (getenv ("HOME"), NULL, NULL);
-        g_return_val_if_fail (file_path != NULL, OUTPUT_PLUGIN_INIT_FAIL);
+        g_return_val_if_fail (file_path != NULL, FALSE);
     }
 
     set_plugin();
     if (plugin->init)
         plugin->init(&file_write_output);
 
-    return OUTPUT_PLUGIN_INIT_FOUND_DEVICES;
+    return TRUE;
+}
+
+static void file_cleanup (void)
+{
+    g_free (file_path);
+    file_path = NULL;
 }
 
 void file_about (void)
@@ -178,7 +164,7 @@ void file_about (void)
 
     audgui_simple_message (& dialog, GTK_MESSAGE_INFO,
      _("About FileWriter-Plugin"),
-     _("FileWriter-Plugin\n\n"
+     "FileWriter-Plugin\n\n"
      "This program is free software; you can redistribute it and/or modify\n"
      "it under the terms of the GNU General Public License as published by\n"
      "the Free Software Foundation; either version 2 of the License, or\n"
@@ -192,7 +178,7 @@ void file_about (void)
      "You should have received a copy of the GNU General Public License\n"
      "along with this program; if not, write to the Free Software\n"
      "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,\n"
-     "USA."));
+     "USA.");
 }
 
 static VFSFile * safe_create (const gchar * filename)
@@ -243,19 +229,18 @@ static gint file_open(gint fmt, gint rate, gint nch)
 
     if (filenamefromtags)
     {
-        gchar *utf8 = tuple_formatter_make_title_string(tuple, aud_get_gentitle_format());
-
+        gchar * utf8 = aud_playlist_entry_get_title (playlist, pos, FALSE);
         string_replace_char (utf8, '/', ' ');
         filename = string_encode_percent (utf8, FALSE);
         g_free (utf8);
     }
     else
     {
-        const gchar * original = strrchr (aud_playlist_entry_get_filename
-         (playlist, pos), '/');
-
+        temp = aud_playlist_entry_get_filename (playlist, pos);
+        gchar * original = strrchr (temp, '/');
         g_return_val_if_fail (original != NULL, 0);
         filename = g_strdup (original + 1);
+        g_free (temp);
 
         if (!use_suffix)
             if ((temp = strrchr(filename, '.')) != NULL)
@@ -275,7 +260,7 @@ static gint file_open(gint fmt, gint rate, gint nch)
 
     if (save_original)
     {
-        directory = g_strdup (aud_playlist_entry_get_filename (playlist, pos));
+        directory = aud_playlist_entry_get_filename (playlist, pos);
         temp = strrchr (directory, '/');
         g_return_val_if_fail (temp != NULL, 0);
         temp[1] = 0;
@@ -318,9 +303,8 @@ static void file_write(void *ptr, gint length)
     samples_written += length / FMT_SIZEOF (input.format);
 }
 
-static gint file_write_output(void *ptr, gint length)
+static void file_drain (void)
 {
-    return vfs_fwrite(ptr, 1, length, output_file);
 }
 
 static void file_close(void)
@@ -331,25 +315,38 @@ static void file_close(void)
     if (output_file != NULL)
         vfs_fclose(output_file);
     output_file = NULL;
+
+    if (tuple)
+    {
+        tuple_free (tuple);
+        tuple = NULL;
+    }
 }
 
 static void file_flush(gint time)
 {
-    file_close();
-    file_open(input.format, input.frequency, input.channels);
+    samples_written = time * (gint64) input.channels * input.frequency / 1000;
+}
+
+static void file_set_written_time (gint time)
+{
+    /* Guesswork:
+     * If time = 0, we are starting a new song and need to open a new file.  If
+     * time > 0, we have already done this, and we are just setting the time
+     * counter. */
+    if (! time)
+    {
+        file_close ();
+        file_open (input.format, input.frequency, input.channels);
+    }
 
     samples_written = time * (gint64) input.channels * input.frequency / 1000;
 }
 
-static void file_pause(short p)
+static void file_pause (gboolean p)
 {
 }
 
-static gint file_playing(void)
-{
-    return 0;
-}
-
 static gint file_get_time (void)
 {
     return samples_written * 1000 / (input.channels * input.frequency);
@@ -357,8 +354,6 @@ static gint file_get_time (void)
 
 static void configure_ok_cb(gpointer data)
 {
-    mcs_handle_t *db;
-
     fileext = gtk_combo_box_get_active(GTK_COMBO_BOX(fileext_combo));
 
     g_free (file_path);
@@ -370,15 +365,12 @@ static void configure_ok_cb(gpointer data)
     prependnumber =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prependnumber_toggle));
 
-    db = aud_cfg_db_open();
-    aud_cfg_db_set_int(db, FILEWRITER_CFGID, "fileext", fileext);
-    aud_cfg_db_set_string(db, FILEWRITER_CFGID, "file_path", file_path);
-    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "save_original", save_original);
-    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "filenamefromtags", filenamefromtags);
-    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "use_suffix", use_suffix);
-    aud_cfg_db_set_bool(db, FILEWRITER_CFGID, "prependnumber", prependnumber);
-
-    aud_cfg_db_close(db);
+    aud_set_int ("filewriter", "fileext", fileext);
+    aud_set_bool ("filewriter", "filenamefromtags", filenamefromtags);
+    aud_set_string ("filewriter", "file_path", file_path);
+    aud_set_bool ("filewriter", "prependnumber", prependnumber);
+    aud_set_bool ("filewriter", "save_original", save_original);
+    aud_set_bool ("filewriter", "use_suffix", use_suffix);
 
     gtk_widget_destroy(configure_win);
     if (path_dirbrowser)
@@ -449,18 +441,15 @@ static void configure_destroy(void)
 
 static void file_configure(void)
 {
-    GtkTooltips *use_suffix_tooltips;
-
     if (!configure_win)
     {
         configure_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
         gtk_window_set_type_hint(GTK_WINDOW(configure_win), GDK_WINDOW_TYPE_HINT_DIALOG);
 
-        gtk_signal_connect(GTK_OBJECT(configure_win), "destroy",
-                           GTK_SIGNAL_FUNC(configure_destroy), NULL);
-        gtk_signal_connect(GTK_OBJECT(configure_win), "destroy",
-                           GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-                           &configure_win);
+        g_signal_connect (configure_win, "destroy", (GCallback)
+         configure_destroy, NULL);
+        g_signal_connect (configure_win, "destroy", (GCallback)
+         gtk_widget_destroyed, & configure_win);
 
         gtk_window_set_title(GTK_WINDOW(configure_win),
                              _("File Writer Configuration"));
@@ -478,16 +467,16 @@ static void file_configure(void)
         fileext_label = gtk_label_new(_("Output file format:"));
         gtk_box_pack_start(GTK_BOX(fileext_hbox), fileext_label, FALSE, FALSE, 0);
 
-        fileext_combo = gtk_combo_box_new_text();
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "WAV");
+        fileext_combo = gtk_combo_box_text_new ();
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "WAV");
 #ifdef FILEWRITER_MP3
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "MP3");
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "MP3");
 #endif
 #ifdef FILEWRITER_VORBIS
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "Vorbis");
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "Vorbis");
 #endif
 #ifdef FILEWRITER_FLAC
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "FLAC");
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "FLAC");
 #endif
         gtk_box_pack_start(GTK_BOX(fileext_hbox), fileext_combo, FALSE, FALSE, 0);
         gtk_combo_box_set_active(GTK_COMBO_BOX(fileext_combo), fileext);
@@ -514,13 +503,11 @@ static void file_configure(void)
 
         saveplace = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(saveplace),
                                                                 _("Save into custom directory"));
-        g_signal_connect(G_OBJECT(saveplace), "toggled", G_CALLBACK(saveplace_custom_cb), NULL);
-        gtk_box_pack_start(GTK_BOX(saveplace_hbox), saveplace, FALSE, FALSE, 0);
-
         if (!save_original)
             gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(saveplace), TRUE);
 
-
+        g_signal_connect(G_OBJECT(saveplace), "toggled", G_CALLBACK(saveplace_custom_cb), NULL);
+        gtk_box_pack_start(GTK_BOX(saveplace_hbox), saveplace, FALSE, FALSE, 0);
 
         path_hbox = gtk_hbox_new(FALSE, 5);
         gtk_box_pack_start(GTK_BOX(configure_vbox), path_hbox, FALSE, FALSE, 0);
@@ -570,9 +557,6 @@ static void file_configure(void)
         use_suffix_toggle = gtk_check_button_new_with_label(_("Don't strip file name extension"));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_suffix_toggle), use_suffix);
         gtk_box_pack_start(GTK_BOX(configure_vbox), use_suffix_toggle, FALSE, FALSE, 0);
-        use_suffix_tooltips = gtk_tooltips_new();
-        gtk_tooltips_set_tip(use_suffix_tooltips, use_suffix_toggle, _("If enabled, the extension from the original filename will not be stripped before adding the new file extension to the end."), NULL);
-        gtk_tooltips_enable(use_suffix_tooltips);
 
         if (filenamefromtags)
             gtk_widget_set_sensitive(use_suffix_toggle, FALSE);
@@ -594,23 +578,40 @@ static void file_configure(void)
         configure_bbox = gtk_hbutton_box_new();
         gtk_button_box_set_layout(GTK_BUTTON_BOX(configure_bbox),
                                   GTK_BUTTONBOX_END);
-        gtk_button_box_set_spacing(GTK_BUTTON_BOX(configure_bbox), 5);
         gtk_box_pack_start(GTK_BOX(configure_vbox), configure_bbox,
                            FALSE, FALSE, 0);
 
         configure_cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-        gtk_signal_connect_object(GTK_OBJECT(configure_cancel), "clicked",
-                                  GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                                  GTK_OBJECT(configure_win));
+        g_signal_connect_swapped (configure_cancel, "clicked", (GCallback)
+         gtk_widget_destroy, configure_win);
         gtk_box_pack_start(GTK_BOX(configure_bbox), configure_cancel,
                            TRUE, TRUE, 0);
 
         configure_ok = gtk_button_new_from_stock(GTK_STOCK_OK);
-        gtk_signal_connect(GTK_OBJECT(configure_ok), "clicked",
-                           GTK_SIGNAL_FUNC(configure_ok_cb), NULL);
+        g_signal_connect (configure_ok, "clicked", (GCallback) configure_ok_cb,
+         NULL);
         gtk_box_pack_start(GTK_BOX(configure_bbox), configure_ok,
                            TRUE, TRUE, 0);
 
         gtk_widget_show_all(configure_win);
     }
 }
+
+AUD_OUTPUT_PLUGIN
+(
+ .name = "FileWriter",
+ .init = file_init,
+ .cleanup = file_cleanup,
+ .about = file_about,
+ .configure = file_configure,
+ .probe_priority = 0,
+ .open_audio = file_open,
+ .close_audio = file_close,
+ .write_audio = file_write,
+ .drain = file_drain,
+ .written_time = file_get_time,
+ .output_time = file_get_time,
+ .pause = file_pause,
+ .flush = file_flush,
+ .set_written_time = file_set_written_time,
+)
