@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2010 William Pitcock <nenolod@dereferenced.org>.
- * Copyright (c) 2010 John Lindgren <john.lindgren@tds.net>.
+ * Copyright (c) 2010-2011 John Lindgren <john.lindgren@tds.net>.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -19,7 +19,10 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#undef DEBUG
+/* must come before <mpg123.h> to get _FILE_OFFSET_BITS */
+#include "config.h"
+
+#include <mpg123.h>
 
 #ifdef DEBUG_MPG123_IO
 # define MPG123_IODBG(...)	AUDDBG(__VA_ARGS__)
@@ -33,41 +36,12 @@
 #include <audacious/plugin.h>
 #include <audacious/audtag.h>
 
-#include "libmpg123/mpg123.h"
-
 /* Define to read all frame headers when calculating file length */
 /* #define FULL_SCAN */
 
-/* id3skip.c */
-gint id3_header_size (const guchar * data, gint size);
-
 static GMutex *ctrl_mutex = NULL;
 static GCond *ctrl_cond = NULL;
 
-/** utility functions **/
-static gboolean mpg123_prefill (mpg123_handle * decoder, VFSFile * handle,
- gint * _result)
-{
-	guchar buffer[16384];
-	gsize length;
-	gint result;
-
-	do
-	{
-		if ((length = vfs_fread (buffer, 1, 16384, handle)) <= 0)
-			return FALSE;
-
-		result = mpg123_decode (decoder, buffer, length, NULL, 0, NULL);
-	}
-	while (result == MPG123_NEED_MORE);
-
-	if (result < 0)
-		AUDDBG ("mpg123 error: %s\n", mpg123_plain_strerror (result));
-
-	* _result = result;
-	return TRUE;
-}
-
 static ssize_t replace_read (void * file, void * buffer, size_t length)
 {
 	return vfs_fread (buffer, 1, length, file);
@@ -79,8 +53,7 @@ static off_t replace_lseek (void * file, off_t to, int whence)
 }
 
 /** plugin glue **/
-static void
-aud_mpg123_init(void)
+static gboolean aud_mpg123_init (void)
 {
 	AUDDBG("initializing mpg123 library\n");
 	mpg123_init();
@@ -88,6 +61,8 @@ aud_mpg123_init(void)
 	AUDDBG("initializing control mutex\n");
 	ctrl_mutex = g_mutex_new();
 	ctrl_cond = g_cond_new();
+
+	return TRUE;
 }
 
 static void
@@ -101,6 +76,17 @@ aud_mpg123_deinit(void)
 	g_cond_free(ctrl_cond);
 }
 
+static void set_format (mpg123_handle * dec)
+{
+	static const gint rates[] = {8000, 11025, 12000, 16000, 22050, 24000, 32000,
+	 44100, 48000};
+
+	mpg123_format_none (dec);
+	for (gint i = 0; i < G_N_ELEMENTS (rates); i ++)
+		mpg123_format (dec, rates[i], MPG123_MONO | MPG123_STEREO,
+		 MPG123_ENC_FLOAT_32);
+}
+
 static void make_format_string (const struct mpg123_frameinfo * info, gchar *
  buf, gint bsize)
 {
@@ -110,10 +96,17 @@ static void make_format_string (const struct mpg123_frameinfo * info, gchar *
 
 static gboolean mpg123_probe_for_fd (const gchar * fname, VFSFile * file)
 {
+	/* MPG123 likes to grab WMA streams, so blacklist anything that starts with
+	 * mms://.  If there are mms:// streams out there carrying MP3, they will
+	 * just have to play in ffaudio.  --jlindgren */
+	if (! strncmp (fname, "mms://", 6))
+		return FALSE;
+
 	mpg123_handle * dec = mpg123_new (NULL, NULL);
 	g_return_val_if_fail (dec, FALSE);
 	mpg123_param (dec, MPG123_ADD_FLAGS, MPG123_QUIET, 0);
 	mpg123_replace_reader_handle (dec, replace_read, replace_lseek, NULL);
+	set_format (dec);
 
 	gint res;
 	if ((res = mpg123_open_handle (dec, file)) < 0)
@@ -124,6 +117,11 @@ ERR:
 		return FALSE;
 	}
 
+#ifdef FULL_SCAN
+	if (mpg123_scan (dec) < 0)
+		goto ERR;
+#endif
+
 RETRY:;
 	glong rate;
 	gint chan, enc;
@@ -134,7 +132,7 @@ RETRY:;
 	if ((res = mpg123_info (dec, & info)) < 0)
 		goto ERR;
 
-	gint16 out[chan * (rate / 10)];
+	gfloat out[chan * (rate / 10)];
 	size_t done;
 	while ((res = mpg123_read (dec, (void *) out, sizeof out, & done)) < 0)
 	{
@@ -165,12 +163,18 @@ static Tuple * mpg123_probe_for_tuple (const gchar * filename, VFSFile * file)
 	mpg123_replace_reader_handle (decoder, replace_read, replace_lseek, NULL);
 
 	if ((result = mpg123_open_handle (decoder, file)) < 0)
-		goto ERROR;
+		goto ERR;
+
+#ifdef FULL_SCAN
+	if (mpg123_scan (decoder) < 0)
+		goto ERR;
+#endif
+
 	if ((result = mpg123_getformat (decoder, & rate, & channels, & encoding)) <
 	 0)
-		goto ERROR;
+		goto ERR;
 	if ((result = mpg123_info (decoder, & info)) < 0)
-		goto ERROR;
+		goto ERR;
 
 	Tuple * tuple = tuple_new_from_filename (filename);
 	make_format_string (& info, scratch, sizeof scratch);
@@ -196,14 +200,14 @@ static Tuple * mpg123_probe_for_tuple (const gchar * filename, VFSFile * file)
 
 	if (! vfs_is_streaming (file))
 	{
-		if (! vfs_fseek (file, 0, SEEK_SET))
-			tag_tuple_read (tuple, file);
+		vfs_rewind (file);
+		tag_tuple_read (tuple, file);
 	}
 
 	return tuple;
 
-ERROR:
-	fprintf (stderr, "mpg123 error: %s\n", mpg123_plain_strerror (result));
+ERR:
+	fprintf (stderr, "mpg123 probe error for %s: %s\n", filename, mpg123_plain_strerror (result));
 	mpg123_delete (decoder);
 	return NULL;
 }
@@ -211,11 +215,11 @@ ERROR:
 typedef struct {
 	VFSFile *fd;
 	mpg123_handle *decoder;
-	mpg123_pars *params;
 	glong rate;
 	gint channels;
 	gint encoding;
 	gint64 seek;
+	gboolean stop;
 	gboolean stream;
 	Tuple *tu;
 } MPG123PlaybackContext;
@@ -244,14 +248,39 @@ update_stream_metadata(VFSFile *file, const gchar *name, Tuple *tuple, gint item
 	return changed;
 }
 
+static Tuple * get_stream_tuple (InputPlayback * p, const gchar * filename,
+ VFSFile * file)
+{
+	Tuple * tuple = mpg123_probe_for_tuple (filename, file);
+	if (! tuple)
+		tuple = tuple_new_from_filename (filename);
+
+	update_stream_metadata (file, "track-name", tuple, FIELD_TITLE);
+	update_stream_metadata (file, "stream-name", tuple, FIELD_ARTIST);
+
+	mowgli_object_ref (tuple);
+	p->set_tuple (p, tuple);
+
+	return tuple;
+}
+
+static void update_stream_tuple (InputPlayback * p, VFSFile * file,
+ Tuple * tuple)
+{
+	if (update_stream_metadata (file, "track-name", tuple, FIELD_TITLE) ||
+	 update_stream_metadata (file, "stream-name", tuple, FIELD_ARTIST))
+	{
+		mowgli_object_ref (tuple);
+		p->set_tuple (p, tuple);
+	}
+}
+
 static gboolean mpg123_playback_worker (InputPlayback * data, const gchar *
  filename, VFSFile * file, gint start_time, gint stop_time, gboolean pause)
 {
+	gboolean error = FALSE;
 	MPG123PlaybackContext ctx;
 	gint ret;
-	gint i;
-	const glong *rates;
-	gsize num_rates;
 	gint bitrate = 0, bitrate_sum = 0, bitrate_count = 0;
 	gint bitrate_updated = -1000; /* >= a second away from any position */
 	struct mpg123_frameinfo fi;
@@ -260,73 +289,65 @@ static gboolean mpg123_playback_worker (InputPlayback * data, const gchar *
 	memset(&ctx, 0, sizeof(MPG123PlaybackContext));
 	memset(&fi, 0, sizeof(struct mpg123_frameinfo));
 
-	AUDDBG("playback worker started for %s\n", data->filename);
-
-	ctx.seek = (start_time > 0) ? start_time : -1;
-	data->data = &ctx;
-
-	AUDDBG("decoder setup\n");
-	mpg123_rates(&rates, &num_rates);
-
-	ctx.params = mpg123_new_pars(&ret);
-	mpg123_par(ctx.params, MPG123_ADD_FLAGS, MPG123_QUIET, 0);
-	mpg123_par(ctx.params, MPG123_ADD_FLAGS, MPG123_GAPLESS, 0);
-	mpg123_par(ctx.params, MPG123_ADD_FLAGS, MPG123_SEEKBUFFER, 0);
-	mpg123_par(ctx.params, MPG123_RVA, MPG123_RVA_OFF, 0);
-
-	ctx.decoder = mpg123_parnew(ctx.params, NULL, &ret);
-	if (ctx.decoder == NULL)
-	{
-		AUDDBG("mpg123 error: %s", mpg123_plain_strerror(ret));
-		mpg123_delete_pars(ctx.params);
-		data->error = TRUE;
-		return FALSE;
-	}
-
-	if ((ret = mpg123_open_feed(ctx.decoder)) != MPG123_OK)
-	{
-		AUDDBG("mpg123 error: %s", mpg123_plain_strerror(ret));
-		mpg123_delete(ctx.decoder);
-		mpg123_delete_pars(ctx.params);
-		data->error = TRUE;
-		return FALSE;
-	}
-
-	AUDDBG("decoder format configuration\n");
-	mpg123_format_none(ctx.decoder);
-	for (i = 0; i < num_rates; i++)
-		mpg123_format (ctx.decoder, rates[i], (MPG123_MONO | MPG123_STEREO),
-		 MPG123_ENC_SIGNED_16);
-
+	AUDDBG("playback worker started for %s\n", filename);
 	ctx.fd = file;
 
-	AUDDBG("checking if stream @%p is seekable\n", ctx.fd);
-	if (vfs_is_streaming(ctx.fd))
-	{
-		AUDDBG(" ... it's not.\n");
-		ctx.stream = TRUE;
-		ctx.tu = mpg123_probe_for_tuple (filename, file);
-	}
-	else
-		AUDDBG(" ... it is.\n");
+	AUDDBG ("Checking for streaming ...\n");
+	g_return_val_if_fail (file, FALSE);
+	ctx.stream = vfs_is_streaming (file);
+	ctx.tu = ctx.stream ? get_stream_tuple (data, filename, file) : NULL;
 
-	AUDDBG("decoder format identification\n");
-	if (! mpg123_prefill (ctx.decoder, ctx.fd, & ret) || ret !=
-	 MPG123_NEW_FORMAT)
+	ctx.seek = (start_time > 0) ? start_time : -1;
+	ctx.stop = FALSE;
+	data->set_data (data, & ctx);
+
+	ctx.decoder = mpg123_new (NULL, NULL);
+	g_return_val_if_fail (ctx.decoder, FALSE);
+	mpg123_param (ctx.decoder, MPG123_ADD_FLAGS, MPG123_QUIET, 0);
+	mpg123_param (ctx.decoder, MPG123_ADD_FLAGS, MPG123_GAPLESS, 0);
+	mpg123_param (ctx.decoder, MPG123_ADD_FLAGS, MPG123_SEEKBUFFER, 0);
+	mpg123_replace_reader_handle (ctx.decoder, replace_read, replace_lseek, NULL);
+	set_format (ctx.decoder);
+
+	if (mpg123_open_handle (ctx.decoder, file) < 0)
 	{
-		data->error = TRUE;
+OPEN_ERROR:
+		fprintf (stderr, "mpg123: Error opening %s: %s.\n", filename,
+		 mpg123_strerror (ctx.decoder));
+		error = TRUE;
 		goto cleanup;
 	}
 
-	mpg123_getformat(ctx.decoder, &ctx.rate, &ctx.channels, &ctx.encoding);
+	gfloat outbuf[8192];
+	size_t outbuf_size = 0;
 
-	AUDDBG("stream identified as MPEG; %ldHz %d channels, encoding %d\n",
-		ctx.rate, ctx.channels, ctx.encoding);
+#ifdef FULL_SCAN
+	if (mpg123_scan (ctx.decoder) < 0)
+		goto OPEN_ERROR;
+#endif
 
-	AUDDBG("opening audio\n");
-	if (! data->output->open_audio (FMT_S16_NE, ctx.rate, ctx.channels))
+GET_FORMAT:
+	if (mpg123_getformat (ctx.decoder, & ctx.rate, & ctx.channels,
+	 & ctx.encoding) < 0)
+		goto OPEN_ERROR;
+
+	while ((ret = mpg123_read (ctx.decoder, (void *) outbuf, sizeof outbuf,
+	 & outbuf_size)) < 0)
 	{
-		data->error = TRUE;
+		if (ret == MPG123_NEW_FORMAT)
+			goto GET_FORMAT;
+		goto OPEN_ERROR;
+	}
+
+	if (mpg123_info (ctx.decoder, & fi) < 0)
+		goto OPEN_ERROR;
+
+	bitrate = fi.bitrate * 1000;
+	data->set_params (data, bitrate, ctx.rate, ctx.channels);
+
+	if (! data->output->open_audio (FMT_FLOAT, ctx.rate, ctx.channels))
+	{
+		error = TRUE;
 		goto cleanup;
 	}
 
@@ -338,16 +359,46 @@ static gboolean mpg123_playback_worker (InputPlayback * data, const gchar *
 	g_mutex_lock(ctrl_mutex);
 
 	AUDDBG("starting decode\n");
-	data->playing = TRUE;
 	data->set_pb_ready(data);
 
 	g_mutex_unlock(ctrl_mutex);
 
-	while (data->playing && (stop_time < 0 || data->output->written_time () <
-	 stop_time))
+	gint64 frames_played = 0;
+	gint64 frames_total = (stop_time - start_time) * ctx.rate / 1000;
+
+	while (1)
 	{
-		gint16 outbuf[ctx.channels * (ctx.rate / 100)];
-		gsize outbuf_size;
+		g_mutex_lock (ctrl_mutex);
+
+		if (ctx.stop)
+		{
+			g_mutex_unlock (ctrl_mutex);
+			break;
+		}
+
+		if (ctx.seek >= 0)
+		{
+			if (mpg123_seek (ctx.decoder, (gint64) ctx.seek * ctx.rate / 1000,
+			 SEEK_SET) < 0)
+			{
+				fprintf (stderr, "mpg123 error in %s: %s\n", filename,
+				 mpg123_strerror (ctx.decoder));
+				ctx.seek = -1;
+				g_cond_signal (ctrl_cond);
+				g_mutex_unlock (ctrl_mutex);
+			}
+			else
+			{
+				data->output->flush (ctx.seek);
+				frames_played = (ctx.seek - start_time) * ctx.rate / 1000;
+				outbuf_size = 0;
+				ctx.seek = -1;
+			}
+
+			g_cond_signal (ctrl_cond);
+		}
+
+		g_mutex_unlock (ctrl_mutex);
 
 		mpg123_info(ctx.decoder, &fi);
 		bitrate_sum += fi.bitrate;
@@ -356,7 +407,7 @@ static gboolean mpg123_playback_worker (InputPlayback * data, const gchar *
 		if (bitrate_sum / bitrate_count != bitrate && abs
 		 (data->output->written_time () - bitrate_updated) >= 1000)
 		{
-			data->set_params(data, NULL, 0, bitrate_sum / bitrate_count * 1000,
+			data->set_params (data, bitrate_sum / bitrate_count * 1000,
 			 ctx.rate, ctx.channels);
 			bitrate = bitrate_sum / bitrate_count;
 			bitrate_sum = 0;
@@ -364,119 +415,58 @@ static gboolean mpg123_playback_worker (InputPlayback * data, const gchar *
 			bitrate_updated = data->output->written_time ();
 		}
 
-		/* deal with shoutcast titles nonsense */
 		if (ctx.stream)
+			update_stream_tuple (data, file, ctx.tu);
+
+		if (! outbuf_size && (ret = mpg123_read (ctx.decoder, (void *) outbuf,
+		 sizeof outbuf, & outbuf_size)) < 0)
 		{
-			gboolean changed = FALSE;
+			if (ret == MPG123_DONE || ret == MPG123_ERR_READER)
+				goto decode_cleanup;
 
-			if (!ctx.tu)
-				ctx.tu = tuple_new_from_filename(data->filename);
-
-			changed = changed || update_stream_metadata(ctx.fd, "track-name", ctx.tu, FIELD_TITLE);
-			changed = changed || update_stream_metadata(ctx.fd, "stream-name", ctx.tu, FIELD_ALBUM);
-
-			if (changed)
-			{
-				mowgli_object_ref(ctx.tu);
-				data->set_tuple(data, ctx.tu);
-			}
-		}
-
-		do
-		{
-			guchar buf[16384];
-			gsize len = 0;
-
-			if (ret == MPG123_NEED_MORE)
-			{
-				MPG123_IODBG("mpg123 requested more data\n");
-
-				len = vfs_fread(buf, 1, 16384, ctx.fd);
-				if (len <= 0)
-				{
-					if (len == 0)
-					{
-						MPG123_IODBG("stream EOF (well, read failed)\n");
-						mpg123_decode (ctx.decoder, NULL, 0, (guchar *) outbuf,
-						 sizeof outbuf, & outbuf_size);
-
-						MPG123_IODBG("passing %ld bytes of audio\n", outbuf_size);
-						data->output->write_audio (outbuf, outbuf_size);
-						data->eof = TRUE;
-						goto decode_cleanup;
-					}
-					else
-						goto decode_cleanup;
-				}
-
-				MPG123_IODBG("got %ld bytes for mpg123\n", len);
-			}
-
-			ret = mpg123_decode (ctx.decoder, buf, len, (guchar *) outbuf,
-			 sizeof outbuf, & outbuf_size);
-			data->output->write_audio (outbuf, outbuf_size);
-		} while (ret == MPG123_NEED_MORE);
-
-		if (ret < 0)
-		{
-			fprintf (stderr, "mpg123 error: %s\n", mpg123_plain_strerror (ret));
+			fprintf (stderr, "mpg123 error in %s: %s\n", filename,
+			 mpg123_strerror (ctx.decoder));
 
 			if (++ error_count >= 10)
 			{
-				data->error = TRUE;
+				error = TRUE;
 				goto decode_cleanup;
 			}
 		}
 		else
+		{
 			error_count = 0;
 
-		g_mutex_lock(ctrl_mutex);
+			gboolean stop = FALSE;
 
-		if (data->playing == FALSE)
-		{
-			g_mutex_unlock(ctrl_mutex);
-			break;
-		}
-
-		if (ctx.seek != -1)
-		{
-			off_t byteoff, sampleoff;
-
-			sampleoff = mpg123_feedseek (ctx.decoder, (gint64) ctx.seek *
-			 ctx.rate / 1000, SEEK_SET, & byteoff);
-			if (sampleoff < 0)
+			if (stop_time >= 0)
 			{
-				fprintf (stderr, "mpg123 error: %s\n", mpg123_strerror (ctx.decoder));
-				ctx.seek = -1;
-				g_cond_signal (ctrl_cond);
-				g_mutex_unlock(ctrl_mutex);
-				continue;
+				gint64 remain = 2 * ctx.channels * (frames_total - frames_played);
+				remain = MAX (0, remain);
+
+				if (outbuf_size >= remain)
+				{
+					outbuf_size = remain;
+					stop = TRUE;
+				}
 			}
 
-			AUDDBG ("seeking to %d (byte %d)\n", (gint) ctx.seek, (gint) byteoff);
-			if (vfs_fseek (ctx.fd, byteoff, SEEK_SET))
-			{
-				g_mutex_unlock (ctrl_mutex);
+			data->output->write_audio (outbuf, outbuf_size);
+			frames_played += outbuf_size / (sizeof outbuf[0] * ctx.channels);
+			outbuf_size = 0;
+
+			if (stop)
 				goto decode_cleanup;
-			}
-
-			data->output->flush (ctx.seek);
-			ctx.seek = -1;
-
-			g_cond_signal(ctrl_cond);
 		}
-
-		g_mutex_unlock(ctrl_mutex);
 	}
 
 decode_cleanup:
-	AUDDBG("eof reached\n");
-	while (data->playing && data->output->buffer_playing())
+	while (data->output->buffer_playing ())
 		g_usleep(10000);
 
 	AUDDBG("decode complete\n");
 	g_mutex_lock (ctrl_mutex);
-	data->playing = FALSE;
+	data->set_data (data, NULL);
 	g_cond_signal (ctrl_cond); /* wake up any waiting request */
 	g_mutex_unlock (ctrl_mutex);
 
@@ -484,43 +474,45 @@ decode_cleanup:
 
 cleanup:
 	mpg123_delete(ctx.decoder);
-	mpg123_delete_pars(ctx.params);
 	if (ctx.tu)
 		tuple_free (ctx.tu);
-	return ! data->error;
+	return ! error;
 }
 
 static void mpg123_stop_playback_worker (InputPlayback * data)
 {
 	g_mutex_lock (ctrl_mutex);
+	MPG123PlaybackContext * context = data->get_data (data);
 
-	if (data->playing)
+	if (context != NULL)
 	{
+		context->stop = TRUE;
 		data->output->abort_write ();
-		data->playing = FALSE;
 		g_cond_signal (ctrl_cond);
 	}
 
 	g_mutex_unlock (ctrl_mutex);
 }
 
-static void mpg123_pause_playback_worker (InputPlayback * data, gshort pause)
+static void mpg123_pause_playback_worker (InputPlayback * data, gboolean pause)
 {
 	g_mutex_lock (ctrl_mutex);
+	MPG123PlaybackContext * context = data->get_data (data);
 
-	if (data->playing)
-        data->output->pause (pause);
+	if (context != NULL)
+		data->output->pause (pause);
 
 	g_mutex_unlock (ctrl_mutex);
 }
 
-static void mpg123_seek_time (InputPlayback * data, gulong time)
+static void mpg123_seek_time (InputPlayback * data, gint time)
 {
 	g_mutex_lock (ctrl_mutex);
+	MPG123PlaybackContext * context = data->get_data (data);
 
-	if (data->playing)
+	if (context != NULL)
 	{
-		((MPG123PlaybackContext *) data->data)->seek = time;
+		context->seek = time;
 		data->output->abort_write ();
 		g_cond_signal (ctrl_cond);
 		g_cond_wait (ctrl_cond, ctrl_mutex);
@@ -546,11 +538,12 @@ static gboolean mpg123_get_image (const gchar * filename, VFSFile * handle,
 /** plugin description header **/
 static const gchar *mpg123_fmts[] = { "mp3", "mp2", "mp1", "bmu", NULL };
 
-static InputPlugin mpg123_ip = {
+AUD_INPUT_PLUGIN
+(
 	.init = aud_mpg123_init,
 	.cleanup = aud_mpg123_deinit,
-	.description = "MPG123",
-	.vfs_extensions = mpg123_fmts,
+	.name = "MPG123",
+	.extensions = mpg123_fmts,
 	.is_our_file_from_vfs = mpg123_probe_for_fd,
 	.probe_for_tuple = mpg123_probe_for_tuple,
 	.play = mpg123_playback_worker,
@@ -559,8 +552,4 @@ static InputPlugin mpg123_ip = {
 	.pause = mpg123_pause_playback_worker,
 	.update_song_tuple = mpg123_write_tag,
 	.get_song_image = mpg123_get_image,
-};
-
-static InputPlugin *mpg123_iplist[] = { &mpg123_ip, NULL };
-
-SIMPLE_INPUT_PLUGIN(mpg123, mpg123_iplist);
+)
