@@ -1,5 +1,5 @@
+#include <inttypes.h>
 #include <pthread.h>
-#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -7,7 +7,6 @@
 #include "fmt.h"
 #include "plugin.h"
 #include "scrobbler.h"
-#include "config.h"
 #include "settings.h"
 #include <glib.h>
 
@@ -490,12 +489,11 @@ static GStaticMutex submit_mutex = G_STATIC_MUTEX_INIT;
 
 gpointer sc_curl_perform_thread(gpointer data)
 {
-    int status;
     CURL *curl = (CURL *) data;
 
     g_static_mutex_lock(&submit_mutex);
 
-    status = curl_easy_perform(curl);
+    curl_easy_perform(curl);
     curl_easy_cleanup(curl);
 
     if (sc_parse_sb_res()) {
@@ -533,15 +531,15 @@ static int sc_handshake(void)
     char *auth_tmp;
     char *auth;
 
-    auth_tmp = g_strdup_printf("%s%ld", sc_password, ts);
+    auth_tmp = g_strdup_printf("%s%"PRId64, sc_password, (gint64) ts);
     auth = (char *)md5_string(auth_tmp, strlen(auth_tmp));
     g_free(auth_tmp);
     hexify(auth, 16);
     auth_tmp = g_strdup(sc_response_hash);
 
-    g_snprintf(buf, sizeof(buf), "%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%ld&a=%s",
+    g_snprintf(buf, sizeof(buf), "%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%"PRId64"&a=%s",
             sc_hs_url, SCROBBLER_VERSION,
-            SCROBBLER_CLI_ID, SCROBBLER_IMPLEMENTATION, sc_username, ts,
+            SCROBBLER_CLI_ID, SCROBBLER_IMPLEMENTATION, sc_username, (gint64) ts,
             auth_tmp);
     g_free(auth_tmp);
 
@@ -916,26 +914,22 @@ static void sc_handlequeue(GMutex *mutex)
 
 static void read_cache(void)
 {
-    FILE *fd;
-    char buf[PATH_MAX];
     int i=0;
     item_t *item;
-    gchar* config_datadir;
 
-    config_datadir = aud_util_get_localdir();
-    g_snprintf(buf, sizeof(buf), "%s/scrobblerqueue.txt", config_datadir);
-    g_free(config_datadir);
+    gchar * path = g_strconcat (aud_get_path (AUD_PATH_USER_DIR), "/scrobblerqueue.txt", NULL);
 
-    if (!(fd = fopen(buf, "r")))
+    if (! g_file_test (path, G_FILE_TEST_EXISTS))
         return;
-    AUDDBG("Opening %s\n", buf);
-    fclose(fd);
+
+    AUDDBG ("Opening %s\n", path);
 
     gchar* cache;
     gchar** values;
     gchar** entry;
-    g_file_get_contents(buf, &cache, NULL, NULL);
+    g_file_get_contents (path, & cache, NULL, NULL);
     values = g_strsplit(cache, "\n", 0);
+    g_free (path);
 
     int x;
     for (x=0; values[x] && strlen(values[x]); x++) {
@@ -994,28 +988,27 @@ static void dump_queue(void)
 {
     FILE *fd;
     item_t *item;
-    char *home, buf[PATH_MAX];
-    gchar* config_datadir;
 
     /*AUDDBG("Entering dump_queue();");*/
 
+    gchar * home;
     if (!(home = getenv("HOME")))
     {
         AUDDBG("No HOME directory found. Cannot dump queue.\n");
         return;
     }
 
-    config_datadir = aud_util_get_localdir();
-    g_snprintf(buf, sizeof(buf), "%s/scrobblerqueue.txt", config_datadir);
-    g_free(config_datadir);
+    gchar * path = g_strconcat (aud_get_path (AUD_PATH_USER_DIR), "/scrobblerqueue.txt", NULL);
 
-    if (!(fd = fopen(buf, "w")))
+    if (! (fd = fopen (path, "w")))
     {
-        AUDDBG("Failure opening %s\n", buf);
+        AUDDBG("Failure opening %s\n", path);
+        g_free (path);
         return;
     }
 
-    AUDDBG("Opening %s\n", buf);
+    AUDDBG("Opening %s\n", path);
+    g_free (path);
 
     q_peekall(1);
 
